@@ -5,7 +5,8 @@
 #'
 #' @exportClass tscopulaU
 #'
-setClassUnion("tscopulaU", c("armacopula", "dvinecopula", "dvinecopula2"))
+setClassUnion("tscopulaU", c("armacopula", "sarmacopula",
+                             "dvinecopula", "dvinecopula2", "dvinecopula3"))
 
 #' Calculate standardized ranks of data
 #'
@@ -143,6 +144,21 @@ setMethod("show", "tscopulafit", function(object) {
         kpacf <- paste(kpacf, "with max lag", modobject@modelspec$maxlag)
       cat("KPACF: ", kpacf,"\n", sep = "")
     }
+    if (is(object@tscopula, "dvinecopula3")) {
+      modobject <- object@tscopula
+      cat("non-Gaussian substitutions:\n")
+      locs <- modobject@modelspec$location
+      cat(" - locations:", locs, "\n", sep = " ")
+      cat(" - families:", modobject@modelspec$family, "\n", sep = " ")
+      tau <- kendall(modobject, lagmax = max(locs))[locs]
+      rot <- ifelse(tau >= 0, modobject@modelspec$posrot, modobject@modelspec$negrot)
+      cat(" - rotations:", rot, "\n", sep = " ")
+      cat(" - Kendall's tau:", round(tau,3), "\n", sep = " ")
+      kpacf  <- modobject@modelspec$kpacf
+      if (modobject@modelspec$maxlag != Inf)
+        kpacf <- paste(kpacf, "with max lag", modobject@modelspec$maxlag)
+      cat("KPACF: ", kpacf,"\n", sep = "")
+    }
   }
   else {
     stop("Unknown tscopula type")
